var fs = require('fs'),
    util = require("util"),
    events = require("events"),
    an = require("an-module"),
    mkdirp = require('mkdirp'),
    crypto = require('crypto');

var an_fs = function (dir) {
    events.EventEmitter.call(this);
    var self = this;
    var base_path = dir + "/filestore";
    raw_path = base_path + "/raw";
    link_path = base_path + "/link";
    
    mkdirp(raw_path, 0775, function (err) {
        if (err)
         console.error(err);

        mkdirp(link_path, 0775, function (err) {
         if (err)
          console.error(err);

                raw_path += "/";
                link_path += "/";
                console.log("initialized fs: ", dir);
                self.emit("ready");
        });
    });
}
util.inherits(an_fs, events.EventEmitter);


an_fs.prototype.put = function (options, cb) {
    var key = options.key;
    var value = options.value;
    cb = cb || function () {};
    md5sum = crypto.createHash('md5');
    var Hvalue = md5sum.update(value).digest('hex');
    fs.writeFile(raw_path + Hvalue, value,function(err) {
     if(err && err.code != "EEXIST")
      throw err;
    });

    fs.stat(link_path + key, function (err, res) {
        if (err || !res.isDirectory()) fs.mkdirSync(link_path + key);

        fs.symlink(raw_path + Hvalue, link_path + key + "/" + Hvalue, function (err) {
        	if(err && err.code != "EEXIST")
      			cb(err);

            cb(null, null);
        });
    });
}

an_fs.prototype.get = function (key, cb) {
    fs.readdir(link_path + key, function (err, files) {
        if (err) {
            cb(err);
            return;
        }
        res = [];

        for (var i = 0, len = files.length; i < len; ++i) {
            res.push({
                key: files[i],
                value: fs.readFileSync(raw_path + files[i], 'utf8')
            });
        }
        cb(null, res);
    });
}
an_fs.prototype.remove = function (options, cb) {
    cb = cb || function () {};
    var key = options.key || null;
    var value = options.value || null;
    if (key !== null) {
        fs.readdir(link_path + key, function (err, files) {
            if (err) {
                cb(err);
                return;
            }


            for (var i = 0, len = files.length; i < len; ++i) {
                fs.unlinkSync(link_path + key + "/" + files[i]);
            }
            fs.rmdirSync(link_path + key);
            cb(null);
        });
    } else // value 
    {
        md5sum = crypto.createHash('md5');
        fs.unlinkSync(raw_path + value); //md5sum.update(value).digest('hex'));
        cb();
    }
}

var an_git = require("./git.js");

// starting the module
an.start(function() {
    // register service
    var storserv = new an.provide.service("storage");
// loading a plain file storage
    var fs = new an_fs(an.config.module_dir);
// loading versioned git storage
    var git = new an_git(an.config.module_dir);
// register public API
    storserv.add_methods({
// store a value under key
        put: function(key,value) {
             fs.put({key:key,value:value}, this.callback);
        },
// get the value for key
        get:function(key) {
             fs.get(key, this.callback);
        },
// remove a value
        remove: function(key,value) {
             fs.remove({key:key,value:value}, this.callback);
        },
// CRD Operations for a "git store" which is a git repository        
        git_createRepo: function(name) {
             git.createRepo(name, this.callback);
        },
        git_getRepo: function(name) {
             git.getRepo(name, this.callback);
        },
        git_removeRepo: function(name) {
             git.removeRepo(name, this.callback);
        },
// update an object inside the repository, this triggers a commit
        git_repo_update: function(repo,key,value) {
             git.repo.update(repo,key,value, this.callback);
        },
// get an object from the repository
        git_repo_get:function(repo,key) {
             git.repo.get(repo,key, this.callback);
        },
// remove a value, triggers a commit
        git_repo_remove: function(repo,key,value) {
             git.repo.remove(repo,key,value, this.callback);
        },
// returns the version of a specific file
        git_repo_head: function(repo,key,value) {
             git.repo.head(repo,key,value, this.callback);
        },
        
        
    });

    storserv.start();

});
